#!BPY

"""
Name: 'MeshAscii (*.mesh.ascii)'
Blender: 248
Group: 'Export'
"""

__author__ = "Dusan Pavlicek"
__email__ = "pavlicd@volny.cz"

from Blender import *
from Blender.Mathutils import *

materialToTextures = {}   # material name -> list of texture names
materialToMeshes = {}     # material name -> list of [object, mesh]
textureAttributes = {}    # texture name -> bitmap filename, UV layer name

warningCounter = 0


def Main():
	Window.FileSelector(Export, 'Export', '*.mesh.ascii')
	return


def Export(filename):
	block = []
	multiFiles = Draw.Create(False)
	block.append(('Multiple files by object', multiFiles))
	if Draw.PupBlock('Export mesh', block) == 0:
		multiFiles = 0

	Window.WaitCursor(1)
	meshObjects = GetVisibleMeshObjects()
	if multiFiles == 0:
		ExportToFile(filename, meshObjects)
	else:
		filenames = GenerateFilenames(filename, meshObjects)
		ExportToFiles(filenames, meshObjects)
	Window.WaitCursor(0)

	global warningCounter
	if warningCounter > 0:
		if warningCounter == 1:
			Draw.PupMenu('There has been 1 warning.')
		else:
			Draw.PupMenu('There have been %d warnings.' % warningCounter)
	return


def GetVisibleMeshObjects():
	global warningCounter
	editmode = Window.EditMode()
	Window.EditMode(0)
	meshObjects = []
	scene = Scene.GetCurrent()
	if scene != None:
		for object in scene.objects:
			if object.type != 'Mesh':
				continue
			visible = False
			for objectLayer in object.layers:
				if objectLayer in scene.layers:
					visible = True
					break
			if not visible:
				continue
			mesh = object.getData(mesh=True)
			if not CheckTriangularFaces(mesh):
				print 'Warning: Skipping mesh ' + mesh.name + ' (non-triangular faces).'
				warningCounter += 1
				continue
			meshObjects.append(object)
	Window.EditMode(editmode)
	return meshObjects


def CheckTriangularFaces(mesh):
	for face in mesh.faces:
		if len(face.verts) != 3:
			return False
	return True


def ExportToFile(filename, meshObjects):
	editmode = Window.EditMode()
	Window.EditMode(0)
	file = open(filename, 'w')
	file.write('1 # bones\n')
	file.write('root\n')
	file.write('-1\n')
	file.write('0 0 0\n')
	file.write(str(len(meshObjects)) + ' # meshes\n')
	for object in meshObjects:
		mesh = object.getData(mesh=True)
		ExportMeshToFile(file, object, mesh)
	file.close()
	Window.EditMode(editmode)
	return


def GenerateFilenames(rootFilename, meshObjects):
	if rootFilename.lower().endswith('.mesh.ascii'):
		rootFilename = rootFilename[:-11]
	filenames = []
	for object in meshObjects:
		filename = rootFilename + '_' + object.name + '.mesh.ascii'
		filenames.append(filename)
	return filenames


def ExportToFiles(filenames, meshObjects):
	editmode = Window.EditMode()
	Window.EditMode(0)
	for i in range(0, len(filenames)):
		object = meshObjects[i]
		mesh = object.getData(mesh=True)
		file = open(filenames[i], 'w')
		file.write('0 # bones\n')
		file.write('1 # mesh\n')
		ExportMeshToFile(file, object, mesh)
		file.close()
	Window.EditMode(editmode)
	return


def ExportMeshToFile(file, object, mesh):
	global warningCounter
	if len(mesh.materials) == 0:
		print 'Warning: Skipping mesh ' + mesh.name + ' (no material).'
		warningCounter += 1
		return
	if len(mesh.materials) > 1:
		print 'Warning: Mesh ' + mesh.name + ' has more than one material.'
		warningCounter += 1
	material = mesh.materials[0]
	if material == None:
		print 'Warning: Skipping mesh ' + mesh.name + ' (no material).'
		warningCounter += 1
		return

	meshUVLayers = mesh.getUVLayerNames()
	usedUVLayers = []
	textureData = []

	for mtex in material.getTextures():
		if mtex == None:
			continue
		texture = mtex.tex
		if texture.type != Texture.Types.IMAGE:
			print 'Warning: Skipping texture ' + texture.name + ' (not a bitmap).'
			warningCounter += 1
			continue
		if mtex.texco != Texture.TexCo.UV:
			print 'Warning: Skipping texture ' + texture.name + ' (no UV mapping).'
			warningCounter += 1
			continue
		image = texture.image
		if image == None:
			print 'Warning: Skipping texture ' + texture.name + ' (no image assigned).'
			warningCounter += 1
			continue
		uvLayer = mtex.uvlayer
		if uvLayer == None or uvLayer == '':
			uvLayer = mesh.renderUVLayer
		if uvLayer == None or uvLayer == '':
			print 'Warning: Skipping texture ' + texture.name + ' (no UV mapping).'
			warningCounter += 1
			continue
		if not uvLayer in meshUVLayers:
			print 'Warning: Skipping texture ' + texture.name + ' (unknown UV mapping ' + uvLayer + ').'
			warningCounter += 1
			continue
		if not uvLayer in usedUVLayers:
			uvLayerID = len(usedUVLayers)
			usedUVLayers.append(uvLayer)
		else:
			uvLayerID = usedUVLayers.index(uvLayer)
		textureData.append([image.filename, uvLayerID])

	for uvLayer in meshUVLayers:
		if uvLayer not in usedUVLayers:
			usedUVLayers.append(uvLayer)

	file.write(object.name + '\n')
	file.write(str(len(usedUVLayers)) + ' # uv layers\n')
	file.write(str(len(textureData)) + ' # textures\n')
	for texFilename, texUVLayerID in textureData:
		file.write(texFilename + '\n')
		file.write(str(texUVLayerID) + ' # uv layer index\n')

	mapVertexIDs = {}
	vertexData = []
	faceVertexIDs = []

	for face in mesh.faces:
		vertexIDs = []
		for i in range(0, 3):
			vertex = face.verts[i]
			texCoords = []
			for uvLayer in usedUVLayers:
				mesh.activeUVLayer = uvLayer
				texCoords.append(face.uv[i])
			if mesh.vertexColors:
				color = [MultVertexColor(face.col[i].r), MultVertexColor(face.col[i].g), MultVertexColor(face.col[i].b), 255]
			else:
				color = [255, 255, 255, 255]
			vertexKey = GenerateVertexKey(vertex, color, texCoords)
			if mapVertexIDs.has_key(vertexKey):
				vertexID = mapVertexIDs[vertexKey]
			else:
				vertexID = len(vertexData)
				vertexData.append([vertex, color, texCoords])
				mapVertexIDs[vertexKey] = vertexID
			vertexIDs.append(vertexID)
		faceVertexIDs.append(vertexIDs)

	file.write(str(len(vertexData)) + ' # vertices\n')
	objectMatrix = object.getMatrix()
	origin = Vector(0, 0, 0) * objectMatrix
	for vertex, color, texCoords in vertexData:
		coord = vertex.co * objectMatrix
		normal = vertex.no * objectMatrix - origin
		file.write(str(coord.x) + ' ' + str(coord.z) + ' ' + str(-coord.y) + '\n')
		file.write(str(normal.x) + ' ' + str(normal.z) + ' ' + str(-normal.y) + '\n')
		file.write(str(color[0]) + ' ' + str(color[1]) + ' ' + str(color[2]) + ' ' + str(color[3]) + '\n')
		for texCoord in texCoords:
			file.write(str(texCoord.x) + ' ' + str(1 - texCoord.y) + '\n')
		file.write('0 0 0 0\n')
		file.write('1 0 0 0\n')

	file.write(str(len(mesh.faces)) + ' # faces\n')
	for i in range(0, len(mesh.faces)):
		face = mesh.faces[i]
		vertexIDs = faceVertexIDs[i]
		file.write(str(vertexIDs[0]) + ' ' + str(vertexIDs[2]) + ' ' + str(vertexIDs[1]) + '\n')
	return


def MultVertexColor(a):
	a = int(round(a * 1.0))
	if a > 255:
		a = 255
	return a


def GenerateVertexKey(vertex, color, texCoords):
	key = str(vertex.co) + str(vertex.no) + str(color)
	for texCoord in texCoords:
		key += str(texCoord)
	return key


Main()
